@@ -1,4 +1,4 @@
-from errors import ObjectNotFound
+from errors import *
 import json
 import requests
 import re
@@ -30,13 +30,14 @@ class BaseCriteria(object):
 
 class BaseObject(dict):
     
+    _pk = 'id'
+    
     def __init__(self, id=None):
     
         self._unsaved_values = set()
         self._transient_values = set()
 
-        if id:
-            self['id'] = id
+        self[self._pk] = id
     
     def __setattr__(self, k, v):
         if k[0] == '_' or k in self.__dict__:
@@ -86,12 +87,32 @@ class BaseModel(BaseObject):
     criteria_class = BaseCriteria
     
     ObjectNotFound = ObjectNotFound
+    RequestError = RequestError
+    AccessError = AccessError
+    MethodError = MethodError
+    IntegrityError = IntegrityError
+    
+    inline_models = dict()
 
     def __str__(self):
         return "%s #%s" % (self.__class__.__name__, self.obj_id())
     
+    def __getitem__(self, k):
+        try:
+            return super(BaseModel, self).__getitem__(k)
+        except KeyError:
+            try:
+                resp = self.__class__.request('get', self.instance_url()+'/'+k)
+                self.load_attr(k, resp)
+                return self[k]
+                
+            except self.RequestError:
+                raise KeyError(
+                    "%r attribute or method not found, available values on this object are: %s" %
+                    (k, ', '.join(self.keys())))
+    
     def obj_id(self):
-        return self.id
+        return self[self._pk]
     
     def instance_url(self):
         return "%s/%s" % (self.__class__.url(), self.obj_id())
@@ -110,7 +131,7 @@ class BaseModel(BaseObject):
 
     @classmethod
     def get_from(cls, values):
-        instance = cls(values.get('id', None))
+        instance = cls(values.get(cls._pk, None))
         instance.reload_from(values)
         return instance
 
@@ -128,9 +149,42 @@ class BaseModel(BaseObject):
         self._transient_values = self._transient_values - set(values)
         
         for k, v in values.iteritems():
-            super(BaseObject, self).__setitem__(k, v)
-                #(k, convert_to_object(v))
-
+            self.load_attr(k, v)
+            
+    def load_attr(self, k, v):
+        if isinstance(v, dict):
+            type = self.inline_models.get(k, BaseModel)
+            value = type.get_from(v)
+        elif isinstance(v, list):
+            type = self.inline_models.get(k, BaseModel)
+            value = []
+            for o in v:
+                value.append(type.get_from(o))
+        else:
+            value = v
+        super(BaseObject, self).__setitem__(k, value)
+    
+    
+    def save(self):
+        if self.obj_id():
+            self.reload_from(self.request('put', self.instance_url(), data=self.serialize()))
+        else:
+            self.reload_from(self.request('post', self.__class__.url(), data=self.serialize()))
+            
+    def delete(self):
+        if self.obj_id():
+            self.reload_from(self.request('delete', self.instance_url()))
+            
+    def serialize(self):
+        params = {}
+        if self._unsaved_values:
+            for k in self._unsaved_values:
+                if k == self._pk:
+                    continue
+                v = getattr(self, k)
+                params[k] = v# if v is not None else ""
+        return params
+            
     @classproperty
     @classmethod
     def find(cls):
@@ -139,10 +193,16 @@ class BaseModel(BaseObject):
     @classmethod
     def request(cls, method, url, params=None, headers={'Content-Type': 'application/json'}, data=None):
         
-        resp = requests.request(method, url, headers=headers, data=data, params=params, timeout=80)
-        if 200 <= resp.status_code < 300:
+        resp = requests.request(method, url, headers=headers, data=json.dumps(data), params=params, timeout=80)
+        if 200 <= resp.status_code < 399:
             return resp.json()
-        elif 400 <= resp.status_code < 500:
+        elif resp.status_code == 400:
+            raise IntegrityError(url)
+        elif resp.status_code == 401:
+            raise AccessError(url)
+        elif resp.status_code == 405:
+            raise MethodError(url)
+        elif 402 <= resp.status_code < 500:
             raise ObjectNotFound(url)
         else:
             raise Exception('API query error (%s - %s): %s %s' % (url, resp.status_code, resp.text, params) )
